/*
 * Copyright 2018 beanit
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package com.beanit.iec61850bean.internal;

import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

public class NamedThreadFactory implements ThreadFactory {

  private final AtomicInteger threadCounter = new AtomicInteger(1);
  private final String namePrefix;

  /**
   * Creates a thread factory with the given pool name as a name prefix. Threads created will have
   * the name {@code <poolName>-thread-<thread-counter>}. A common pool name is of format {@code
   * <pool-name>-<pool-id>}.
   *
   * @param poolName the thread pool name
   */
  public NamedThreadFactory(String poolName) {
    this.namePrefix = poolName + "-thread-";
  }

  @Override
  public Thread newThread(Runnable r) {
    Thread thread = new Thread(r);
    thread.setName(namePrefix + threadCounter.getAndIncrement());
    return thread;
  }
}
